/*	$Kolabore: err.c,v 1.3 2006/10/29 16:28:50 maxx Exp $	*/
/*	$OpenBSD: verr.c,v 1.8 2005/08/08 08:05:34 espie Exp $ */
/*	$OpenBSD: verrx.c,v 1.8 2005/08/08 08:05:34 espie Exp $ */
/*	$OpenBSD: vwarn.c,v 1.8 2005/08/08 08:05:34 espie Exp $ */
/*	$OpenBSD: vwarnx.c,v 1.8 2005/08/08 08:05:34 espie Exp $ */
/*	$OpenBSD: err.c,v 1.9 2005/08/08 08:05:33 espie Exp $ */
/*	$OpenBSD: errx.c,v 1.8 2005/08/08 08:05:34 espie Exp $ */
/*	$OpenBSD: warn.c,v 1.8 2005/08/08 08:05:34 espie Exp $ */
/*	$OpenBSD: warnx.c,v 1.7 2005/08/08 08:05:34 espie Exp $ */
/*-
 * Copyright (c) 1993
 *      The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <errno.h>
#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "xsicompat.h"

extern char *__progname;		/* Program name, from crt0. */

__dead void	xsi_verr(int, const char *, va_list);
__dead void	xsi_verrx(int, const char *, va_list);
void		xsi_vwarn(const char *, va_list);
void		xsi_vwarnx(const char *, va_list);

void
xsi_err(int eval, const char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);
	xsi_verr(eval, fmt, ap);
	va_end(ap);
}

void
xsi_errx(int eval, const char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);
	xsi_verrx(eval, fmt, ap);
	va_end(ap);
}

void
xsi_warn(const char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);
	xsi_vwarn(fmt, ap);
	va_end(ap);
}

void
xsi_warnx(const char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);
	xsi_vwarnx(fmt, ap);
	va_end(ap);
}

void
xsi_verr(int eval, const char *fmt, va_list ap)
{
	int sverrno;

	sverrno = errno;
	(void)fprintf(stderr, "%s: ", __progname);
	if (fmt != NULL) {
		(void)vfprintf(stderr, fmt, ap);
		(void)fprintf(stderr, ": ");
	}
	(void)fprintf(stderr, "%s\n", strerror(sverrno));
	exit(eval);
}

void
xsi_verrx(int eval, const char *fmt, va_list ap)
{
	(void)fprintf(stderr, "%s: ", __progname);
	if (fmt != NULL)
		(void)vfprintf(stderr, fmt, ap);
	(void)fprintf(stderr, "\n");
	exit(eval);
}

void
xsi_vwarn(const char *fmt, va_list ap)
{
	int sverrno;

	sverrno = errno;
	(void)fprintf(stderr, "%s: ", __progname);
	if (fmt != NULL) {
		(void)vfprintf(stderr, fmt, ap);
		(void)fprintf(stderr, ": ");
	}
	(void)fprintf(stderr, "%s\n", strerror(sverrno));
}

void
xsi_vwarnx(const char *fmt, va_list ap)
{
	(void)fprintf(stderr, "%s: ", __progname);
	if (fmt != NULL)
		(void)vfprintf(stderr, fmt, ap);
	(void)fprintf(stderr, "\n");
}
