/*	$Kolabore: xsicompat.h,v 1.13 2006/10/29 16:48:57 maxx Exp $	*/
/*
 * Copyright (c) 2006 Moritz Grimm <gtgbr@gmx.net>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef __XSICOMPAT_H__
#define __XSICOMPAT_H__

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif


#ifndef PRIi64
#  define PRIi64 "lld"
#endif /* PRIi64 */

#ifndef PRIu64
#  define PRIu64 "llu"
#endif /* PRIu64 */


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/* Try to get __dead from <sys/cdefs.h> in case it isn't there already */
#ifdef HAVE_SYS_CDEFS_H
#  include <sys/cdefs.h>
#endif

/* Code related to __dead adapted from OpenBSD <sys/cdefs.h> */

#if !defined(__dead)

/*
 * Macro to test if we're using a specific version of gcc or later.
 */
#if defined(__GNUC__) && defined(__GNUC_MINOR__)
#  undef __GNUC_PREREQ__
#  define __GNUC_PREREQ__(ma, mi) \
	((__GNUC__ > (ma)) || (__GNUC__ == (ma) && __GNUC_MINOR__ >= (mi)))
#else
#  define __GNUC_PREREQ__(ma, mi) 0
#endif

#if !__GNUC_PREREQ__(2, 5)
#  define __attribute__(x)	/* delete __attribute__ if non-gcc or gcc1 */
#  if defined(__GNUC__) && !defined(__STRICT_ANSI__)
#    define __dead	__volatile
#  elif defined(lint)
#    define __dead	/* NORETURN */
#  endif
#elif !defined(__STRICT_ANSI__)
#  define __dead	__attribute__((__noreturn__))
#else
#  define __dead
#endif

#endif /* !__dead */

#ifndef HAVE_ERR
#  define err xsi_err
#  define errx xsi_errx
#  define warn xsi_warn
#  define warnx xsi_warnx
#endif /* HAVE_ERR */
__dead void	xsi_err(int, const char *, ...);
__dead void	xsi_errx(int, const char *, ...);
void		xsi_warn(const char *, ...);
void		xsi_warnx(const char *, ...);

#ifndef HAVE_DAEMON
#  define daemon xsi_daemon
#endif /* HAVE_DAEMON */
int		xsi_daemon(int, int);

#ifndef HAVE_STRLCAT
#  define strlcat xsi_strlcat
#endif /* HAVE_STRLCAT */
size_t		xsi_strlcat(char *, const char *, size_t);

#ifndef HAVE_STRLCPY
#  define strlcpy xsi_strlcpy
#endif /* HAVE_STRLCPY */
size_t		xsi_strlcpy(char *, const char *, size_t);

#ifndef HAVE_STRTONUM
#  define strtonum xsi_strtonum
#endif /* HAVE_STRTONUM */
long long	xsi_strtonum(const char *, long long, long long, const char **);


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __XSICOMPAT_H__ */
