/*	$Kolabore: freebsd.h,v 1.23 2006/10/29 18:25:00 maxx Exp $	*/
/*
 * Copyright (c) 2006 Joshua Elsasser <josh@elsasser.org>
 *                    Moritz Grimm <gtgbr@gmx.net>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef __FREEBSD_H__
#define __FREEBSD_H__

#include <cstdarg>

#include "common.h"

using namespace std;

struct freebsd_private;

class XSI : public XSICommon {
public:
	XSI(void);
	~XSI(void);

	int		tryOpenPrivilegedDevices(void);

private:
	void		collectSysInfo(void);
	void		updateStats(void);

	void		setCPUTimes(void);
	void		setMem(void);
	void		setVers(void);
	void		setFilesystems(void);
	void		setDisks(void);
	void		setInterrupts(void);
	void		setNetStats(void);
	void		setPFStats(void);
	void		setSensors(void);

	void		getNetLLAddr(XSI_NetIfStat &, struct sockaddr_dl *);
	void		getNetIfInfo(XSI_NetIfStat &, struct ifaddrs *,
				     struct sockaddr_dl *sdl);
	void		getNetIfmediaInfo(XSI_NetIfStat &);
	void		getNetIfAddrs(XSI_NetIfStat &, struct ifaddrs *, string);
	void		getDefaultRoutes(void);

	u_int		getmib(int *, va_list);
	int		sysctlStr(char**, ...);
	int		sysctlFixed(void*, size_t, ...);
	int		sysctlFixed(void*, size_t, const char*);
	char *		sysctlDynamic(const char *, size_t *);
	string		getMediaWordString(int, int);

	struct freebsd_private *priv;
	int		pf_fd;
};

#endif /* __FREEBSD_H__ */
