/*	$Kolabore: linux.h,v 1.33 2006/10/14 13:34:47 maxx Exp $	*/
/*
 * Copyright (c) 2006 Moritz Grimm <gtgbr@gmx.net>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef __LINUX_H__
#define __LINUX_H__

#include "typesetup.h"

extern "C" {
#include <linux/types.h>
#include <linux/netlink.h>
#include <netinet/in.h>
#include <stdint.h>
}

#include "common.h"

enum linux_sensor_type {
	VOLT,
	TEMP,
	FAN,
	CPUVOLT,
	AMP,
	OTHER
};

struct linux_sensor_private {
	string		driver;
	string		name;
	enum linux_sensor_type type;
	int64_t 	value;
	int		id;
	int		alarm;
	bool		has_fault;
};

using namespace std;

class XSI : public XSICommon {
public:
	XSI(void);
	~XSI(void);

	int		tryOpenPrivilegedDevices(void);

private:
	void		collectSysInfo(void);
	void		updateStats(void);

	void		linuxInit(void);

	void		readSystemNumCPUs(void);
	void		readSystemOSInfo(void);

	void		readNetIfInfo(XSI_NetIfStat &, struct ifaddrs *);
	void		readNetIfAddrs(XSI_NetIfStat &, struct ifaddrs *, string);
	void		readNetIfCounters(XSI_NetIfStat &, vector<string>, string);

	void		updateCPUStats(void);
	void		updateSystemCPUFreq(void);
	void		updateSystemSysInfo(void);
	void		updateLoadAvg(void);
	void		updateInterrupts(void);
	void		updateSensors(void);
	void		updateFSStats(void);
	void		updateDiskStats(void);
	void		updateNetStats(void);
	void		updateDefaultRoutes(void);
	void		updateNetfilter(void);

	uint8_t *	getNetlinkMessage(int, size_t &);
	int		parseRoutes(uint8_t *, size_t, struct in_addr *);
	vector<struct linux_sensor_private> getSensors(vector<string>);
	enum linux_sensor_type getSensorType(string);
	void		getSensorID(string, string &, int &);
	string		getSysfsDeviceName(const char *);
	string		getDirName(const char *);

	int		num_cpu;
	int64_t       **last_cp_time, **cur_cp_time;
	int		rtnl_fd;
};

#endif /* __LINUX_H__ */
