/*	$Kolabore: netbsd.h,v 1.12 2006/10/19 20:03:56 maxx Exp $	*/
/*
 * Copyright (c) 2006 Moritz Grimm <gtgbr@gmx.net>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef __NETBSD_H__
#define __NETBSD_H__

#include <cstdarg>

#include "common.h"

using namespace std;

struct netbsd_private;

class XSI : public XSICommon {
public:
	XSI(void);
	~XSI(void);

	int		tryOpenPrivilegedDevices(void);

private:
	void		collectSysInfo(void);
	void		updateStats(void);

	void		netbsdInit(void);
	string		getSysctlString(int, int, int, unsigned int,
					bool, const char *);

	void		readSystemArch(void);
	void		readSystemModel(void);
	void		readSystemNumCPUs(void);
	void		readSystemOSInfo(void);

	void		updateCPUFreq(void);
	void		updateCPUStats(void);
	void		updateVMStats(void);
	void		updateFSStats(void);
	void		updateDiskStats(void);
	void		updateOSUptime(void);
	void		updateLoadAvg(void);
	void		updateInterrupts(void);
	void		updateSensors(void);
	void		updateNetStats(void);
	void		updatePFStats(void);

	void		getNetLLAddr(XSI_NetIfStat &, struct sockaddr_dl *);
	void		getNetIfInfo(XSI_NetIfStat &, struct ifaddrs *);
	void		getNetIfmediaInfo(XSI_NetIfStat &);
	void		getNetIfAddrs(XSI_NetIfStat &, struct ifaddrs *, string);
	void		getDefaultRoutes(void);

	u_int		getmib(int *, va_list);
	int		sysctlStr(char**, ...);
	string		getMediaWordString(int, int);

	struct netbsd_private *priv;
	int		pf_fd;
};

#endif /* __NETBSD_H__ */
