/*	$Kolabore: netifstat.h,v 1.15 2006/10/29 18:42:25 maxx Exp $	*/
/*
 * Copyright (c) 2006 Moritz Grimm <gtgbr@gmx.net>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * netifstat.h
 * Comprehensive information about a network interface and its statistics.
 */

#ifndef __NETIFSTAT_H__
#define __NETIFSTAT_H__

#include "util.h"

using namespace std;

enum if_link_status {
	IF_LINK_ACTIVE,
	IF_LINK_NOCARRIER,
	IF_LINK_UNKNOWN
};

enum xsi_carp_state {
	CARP_UNSPEC,
	CARP_INIT,
	CARP_BACKUP,
	CARP_MASTER,
	CARP_UNKNOWN
};

class XSI_IfAddress : private Util {
public:
	XSI_IfAddress(void);

	/* For AF_INET and AF_INET6 only: */
	void		setAddress(struct sockaddr *	/* address */,
				   size_t		/* address size */,
				   struct sockaddr *	/* netmask */,
				   int			/* address family */);
	/* Generic: */
	void		setAddressString(const char * /* address */ , int /* AF */);

	bool		hasPrefixLen(void);
	const char *	getAddressString(void);
	const char *	getAddressFamilyString(void);
	const char *	getPrefixLenString(void);

private:
	int		getPrefixLen(void *, int);

	string		address;
	int		family, prefixlen;
};

class XSI_NetIfStat : private Util {
public:
	XSI_NetIfStat(const char * /* interface name */, bool /* is up */);

	void		setIsUp(bool);
	void		setIsRunning(bool);
	void		setIsPromiscuous(bool);
	void		setIsLoopback(bool);
	void		setDescription(const char *);
	void		setMediaString(const char *);
	void		setLladdrString(const char *);
	void		setStatusString(const char *);
	void		setMetric(int);
	void		setMTU(int);
	void		setLinkStatus(enum if_link_status);
	void		setPacketsIn(uint64_t);
	void		setPacketsOut(uint64_t);
	void		setPacketsDropped(uint64_t);
	void		setErrorsIn(uint64_t);
	void		setErrorsOut(uint64_t);
	void		setBytesIn(uint64_t);
	void		setBytesOut(uint64_t);
	void		setCollisions(uint64_t);
	void		setIfAddresses(vector<XSI_IfAddress>);
	void		setPTPDestAddress(struct sockaddr *, size_t);
	void		setCARPStatus(const char * /* carpdev */,
				      enum xsi_carp_state /* status */,
				      int /* vhid */, int /* advskew */,
				      int /* advbase */, bool /* preempt */,
				      bool /* arpbalance */);

	bool		isUp(void);
	bool		isRunning(void);
	bool		isPromiscuous(void);
	bool		isLoopback(void);
	bool		isPointToPoint(void);
	bool		isCARPDev(void);
	bool		hasMetric(void);
	bool		hasMTU(void);
	bool		hasAddresses(void);
	bool		hasDroppedPkgs(void);
	bool		hasErrors(void);
	bool		hasCollisions(void);
	bool		hasTraffic(void);
	const char *	getIfName(void);
	const char *	getDescriptionString(void);
	const char *	getMediaString(void);
	const char *	getLladdrString(void);
	const char *	getStatusString(void);
	const char *	getMetricString(void);
	const char *	getMTUString(void);
	const char *	getLinkStatusString(void);
	const char *	getPacketsInString(void);
	const char *	getPacketsOutString(void);
	const char *	getPacketsDroppedString(void);
	const char *	getErrorsInString(void);
	const char *	getErrorsOutString(void);
	const char *	getBytesInString(void);
	const char *	getBytesOutString(void);
	const char *	getCollisionsString(void);
	const char *	getPacketsInPerSecString(void);
	const char *	getPacketsOutPerSecString(void);
	const char *	getPacketsDroppedPerSecString(void);
	const char *	getBytesInPerSecString(void);
	const char *	getBytesOutPerSecString(void);
	const char *	getPTPDestAddressString(void);
	vector<XSI_IfAddress> getIfAddresses(void);
	const char *	getCARPDeviceString(void);
	const char *	getCARPStatusString(void);
	const char *	getCARPVHIDString(void);
	const char *	getCARPAdvskewString(void);
	const char *	getCARPAdvbaseString(void);
	const char *	getCARPPreemptString(void);
	const char *	getCARPArpbalanceString(void);

private:
	bool			is_up, is_running, is_promiscuous;
	bool			is_loopback, is_pointtopoint;
	string			name;
	string			description;
	string			media;
	string			lladdr;
	string			status;
	int			metric;
	int			mtu;
	enum if_link_status	link_status;
	uint64_t		packets_in, last_packets_in;
	uint64_t		packets_out, last_packets_out;
	uint64_t		packets_dropped, last_packets_dropped;
	uint64_t		errors_in;
	uint64_t		errors_out;
	uint64_t		bytes_in, last_bytes_in;
	uint64_t		bytes_out, last_bytes_out;
	uint64_t		collisions;
	vector<XSI_IfAddress>	addresses;
	string			ptp_dest_addr;
	bool			is_carpdev;
	string			carp_carpdev;
	enum xsi_carp_state	carp_status;
	int			carp_vhid, carp_advskew, carp_advbase;
	int			carp_preempt, carp_arpbalance;
};

#endif /* __NETIFSTAT_H__ */
