/*	$Kolabore: raid.h,v 1.10 2006/10/08 15:54:16 maxx Exp $	*/
/*
 * Copyright (c) 2006 Moritz Grimm <gtgbr@gmx.net>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * raid.h
 * Interface to SCSI RAID, of similar design as sensor support.
 */

#ifndef __RAID_H__
#define __RAID_H__

#include "util.h"

using namespace std;

enum RaidVolumeStatus {
	RAID_VOL_ONLINE,
	RAID_VOL_OFFLINE,
	RAID_VOL_DEGRADED,
	RAID_VOL_BUILDING,
	RAID_VOL_SCRUBBING,
	RAID_VOL_REBUILD,
	RAID_VOL_INVALID,
	RAID_VOL_UNKNOWN
};

enum RaidDiskStatus {
	RAID_DSK_ONLINE,
	RAID_DSK_OFFLINE,
	RAID_DSK_FAILED,
	RAID_DSK_REBUILD,
	RAID_DSK_HOTSPARE,
	RAID_DSK_UNUSED,
	RAID_DSK_SCRUBBING,
	RAID_DSK_INVALID,
	RAID_DSK_UNKNOWN
};

class RDisk : private Util {
public:
	RDisk(void);

	void		setDiskID(int);
	void		setDiskLocation(int, int);
	void		setDiskLUN(int);
	void		setDiskStatus(enum RaidDiskStatus);
	void		setDiskSize(uint64_t);
	void		setDiskEnclosureDevName(const char *);
	void		setDiskVendorName(const char *);
	void		setDiskSerial(const char *);
	const char *	getDiskIDString(void);
	const char *	getDiskBusString(void);
	const char *	getDiskTargetString(void);
	const char *	getDiskLUNString(void);
	const char *	getDiskStatusString(void);
	const char *	getDiskSizeString(void);
	const char *	getDiskEnclosureDevName(void);
	const char *	getDiskVendorName(void);
	const char *	getDiskSerial(void);

private:
	int		id;
	int		bus, target, lun;
	enum RaidDiskStatus status;
	uint64_t	size;
	string		enclosure;
	string		vendor;
	string		serial;
};

class RVolume : private Util {
public:
	RVolume(void);

	void		setVolumeDevName(const char *);
	void		setVolumeID(int);
	void		setRaidLevel(int);
	void		setVolumeStatus(enum RaidVolumeStatus);
	void		setVolumeOpProgress(int);
	void		setVolumeOpElapsed(int);
	void		setVolumeSize(uint64_t);
	void		setVolumeDisks(vector<RDisk>);
	const char *	getVolumeDevName(void);
	const char *	getVolumeIDString(void);
	const char *	getRaidLevelString(void);
	const char *	getVolumeStatusString(void);
	const char *	getVolumeOpProgressString(void);
	const char *	getVolumeOpElapsedString(void);
	const char *	getVolumeSizeString(void);
	vector<RDisk>	getVolumeDisks(void);

private:
	string		device;
	int		id;
	int		level;
	enum RaidVolumeStatus status;
	int		progress;
	int		elapsed;
	uint64_t	size;
	vector<RDisk>	disks;
};

class Raid {
public:
	Raid(const char * /* controller name */);

	void		setVolumes(vector<RVolume>);
	void		setReserveDisks(vector<RDisk>);
	vector<RVolume> getVolumes(void);
	vector<RDisk>	getReserveDisks(void);
	const char *	getRaidControllerName(void);

private:
	vector<RVolume> volumes;
	vector<RDisk>	disks;
	string		controller;
};

#endif /* __RAID_H__ */
