/*	$Kolabore: storage.cpp,v 1.8 2006/10/24 21:24:41 maxx Exp $	*/
/*
 * Copyright (c) 2006 Moritz Grimm <gtgbr@gmx.net>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif
#include "typesetup.h"

extern "C" {
#include <sys/types.h>

#ifdef HAVE_STDINT_H
#  include <stdint.h>
#endif
}

#include <string>
#include <vector>

#include "storage.h"
#include "timekeeper.h"
#include "xmlsysinfo.h"

using namespace std;

XSI_DiskIO::XSI_DiskIO(void)
{
	bytes_written = last_bytes_written = 0;
	bytes_read = last_bytes_read = 0;
}

XSI_DiskIO::XSI_DiskIO(const char *name)
{
	disk_name = name;
	bytes_written = last_bytes_written = 0;
	bytes_read = last_bytes_read = 0;
}

void
XSI_DiskIO::setDiskName(const char *name)
{
	disk_name = name;
}

void
XSI_DiskIO::setBytesWritten(uint64_t writtenb)
{
	last_bytes_written = bytes_written;
	bytes_written = writtenb;
}

void
XSI_DiskIO::setBytesRead(uint64_t readb)
{
	last_bytes_read = bytes_read;
	bytes_read = readb;
}

bool
XSI_DiskIO::isIdle(void)
{
	return (getBytesWriteBPS() == 0 &&
		getBytesReadBPS() == 0);
}

const char *
XSI_DiskIO::getDiskName(void)
{
	return (disk_name.c_str());
}

const char *
XSI_DiskIO::getBytesWrittenString(void)
{
	return (makeIntStr(bytes_written));
}

const char *
XSI_DiskIO::getBytesReadString(void)
{
	return (makeIntStr(bytes_read));
}

uint64_t
XSI_DiskIO::getBytesWritten(void)
{
	return (bytes_written);
}

uint64_t
XSI_DiskIO::getBytesRead(void)
{
	return (bytes_read);
}

const char *
XSI_DiskIO::getBytesWriteBPSString(void)
{
	return (makeFloatStr(getBytesWriteBPS(), 1));
}

const char *
XSI_DiskIO::getBytesReadBPSString(void)
{
	return (makeFloatStr(getBytesReadBPS(), 1));
}

double
XSI_DiskIO::getBytesWriteBPS(void)
{
	uint64_t value;

	if (!bytes_written || XSI_TimeKeeper::get()->timeDelta() < 0.01 || !last_bytes_written)
		return (0);

	value = bytes_written - last_bytes_written;

	if (value == 0)
		return (0);
	else
		return ((double)value / XSI_TimeKeeper::get()->timeDelta());
}

double
XSI_DiskIO::getBytesReadBPS(void)
{
	uint64_t value;

	if (!bytes_read || XSI_TimeKeeper::get()->timeDelta() < 0.01 || !last_bytes_read)
		return (0);

	value = bytes_read - last_bytes_read;

	if (value == 0)
		return (0);
	else
		return ((double)value / XSI_TimeKeeper::get()->timeDelta());
}


XSI_Storage::XSI_Storage(void)
{
	read_bps_total = write_bps_total = 0;
}

void
XSI_Storage::setFilesystems(vector<struct fs> f)
{
	filesystems = f;
}

vector<struct fs>
XSI_Storage::getFilesystems(void)
{
	return (filesystems);
}

void
XSI_Storage::setDisks(vector<XSI_DiskIO> d)
{
	disks = d;
}

vector<XSI_DiskIO>
XSI_Storage::getDisks(void)
{
	return (disks);
}

const char *
XSI_Storage::getreadSpeedTotalString(void)
{
	updateIOTotals();
	return (makeFloatStr(read_bps_total, 1));
}

const char *
XSI_Storage::getwriteSpeedTotalString(void)
{
	updateIOTotals();
	return (makeFloatStr(write_bps_total, 1));
}

void
XSI_Storage::updateIOTotals(void)
{
	vector<XSI_DiskIO>::iterator it;

	read_bps_total = write_bps_total = 0;
	for (it = disks.begin(); it != disks.end(); it++) {
		write_bps_total += it->getBytesWriteBPS();
		read_bps_total += it->getBytesReadBPS();
	}
}
