/*	$Kolabore: storage.h,v 1.7 2006/10/24 21:24:41 maxx Exp $	*/
/*
 * Copyright (c) 2006 Moritz Grimm <gtgbr@gmx.net>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * storage.h
 * Disks, filesystems, etc.
 */

#ifndef __STORAGE_H__
#define __STORAGE_H__

#include "raid.h"
#include "util.h"

using namespace std;

/*
 * struct fs:
 * structure describing a filesystem
 */
struct fs {
	string		mountpoint;	// e.g. /usr
	string		fs_name;	// e.g. /dev/wd0d
	string		fs_type;	// e.g. ffs
	uint64_t	size_total;	// total space (bytes)
	uint64_t	size_free;	// free space (total)
	int64_t		size_avail;	// free space (actual, non-reserved
					//	      available space)
	uint64_t	nodes_total;	// file nodes / inodes
	uint64_t	nodes_free;
};

class XSI_DiskIO : private Util {
public:
	XSI_DiskIO(void);
	XSI_DiskIO(const char * /* disk name */);

	void		setDiskName(const char *);
	void		setBytesWritten(uint64_t);
	void		setBytesRead(uint64_t);

	bool		isIdle(void);
	const char *	getDiskName(void);
	const char *	getBytesWrittenString(void);
	const char *	getBytesReadString(void);
	uint64_t	getBytesWritten(void);
	uint64_t	getBytesRead(void);
	const char *	getBytesWriteBPSString(void);
	const char *	getBytesReadBPSString(void);
	double		getBytesWriteBPS(void);
	double		getBytesReadBPS(void);

private:
	string		disk_name;
	uint64_t	bytes_written, last_bytes_written;
	uint64_t	bytes_read, last_bytes_read;
};

/*
 * XSI_Storage class
 * Gets and outputs information about filesystems and disks
 *   MANDATORY: XSI needs to know about at least one filesystem
 *    OPTIONAL: Local disks, CD-ROMs, etc.
 */
class XSI_Storage : private Util {
public:
	XSI_Storage(void);

	void			setFilesystems(vector<struct fs>);
	void			setDisks(vector<XSI_DiskIO>);

	vector<struct fs>	getFilesystems(void);
	vector<XSI_DiskIO>	getDisks(void);
	const char *		getreadSpeedTotalString(void);
	const char *		getwriteSpeedTotalString(void);

private:
	void			updateIOTotals(void);

	double			read_bps_total;
	double			write_bps_total;
	vector<struct fs>	filesystems;
	vector<XSI_DiskIO>	disks;
};

#endif /* __STORAGE_H__ */
