/*	$Kolabore: timekeeper.cpp,v 1.8 2006/10/03 09:34:31 maxx Exp $	*/
/*
 * Copyright (c) 2006 Moritz Grimm <gtgbr@gmx.net>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif
#include "typesetup.h"

extern "C" {
#include <sys/time.h>
#include <sys/types.h>

#ifdef HAVE_ERR_H
#  include <err.h>
#endif
#ifdef HAVE_STDINT_H
#  include <stdint.h>
#endif
}

#include "timekeeper.h"
#include "compat/xsicompat.h"

using namespace std;

XSI_TimeKeeper *
XSI_TimeKeeper::get(void)
{
	static XSI_TimeKeeper instance;
	return (&instance);
}

bool
XSI_TimeKeeper::haveTimeDelta(void)
{
	return (have_time_delta);
}

double
XSI_TimeKeeper::timeDelta(void)
{
	return (update_time_delta);
}

const char *
XSI_TimeKeeper::updateTimestampString(void)
{
	double stamp;

	stamp = update_time.tv_sec;
	stamp += (double)update_time.tv_usec / (double)1000000;

	return (makeFloatStr(stamp, 1, 0));
}

XSI_TimeKeeper::XSI_TimeKeeper(void)
{
	have_time_delta = false;
	last_update_time.tv_sec = 0;
	last_update_time.tv_usec = 0;
	if (gettimeofday(&update_time, NULL) == -1)
		err(1, "%s: gettimeofday", __func__);
	update_time_delta = 1;
}

void
XSI_TimeKeeper::triggerUpdate(void)
{
	static bool firstrun = true;

	if (firstrun) {
		firstrun = false;
		return;
	}

	last_update_time.tv_sec = update_time.tv_sec;
	last_update_time.tv_usec = update_time.tv_usec;
	if (gettimeofday(&update_time, NULL) == -1)
		err(1, "%s: gettimeofday", __func__);
	update_time_delta =
		((double)update_time.tv_sec + (double)(update_time.tv_usec) / 1000000.0)
		- ((double)last_update_time.tv_sec + (double)(last_update_time.tv_usec) / 1000000.0);
	have_time_delta = true;
}
