/*	$Kolabore: xmlsysinfo.h,v 1.28 2006/10/24 22:10:27 maxx Exp $	*/
/*
 * Copyright (c) 2006 Moritz Grimm <gtgbr@gmx.net>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef __XMLSYSINFO_H__
#define __XMLSYSINFO_H__

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

extern "C" {
#include <libxml/tree.h>
}

#include "cfg.h"
#include "system.h"
#include "network.h"
#include "sensor.h"
#include "storage.h"
#include "util.h"

/*
 * XML Schema version; only loosely coupled with XSI versioning:
 * *.98 == alpha, nothing is fixed
 * *.99 == beta, Schema fixed unless absolutely necessary
 * *.00 == release, Schema must not change
 *
 * The XSD URI may be defined elsewhere, e.g. to point to a local copy.
 */
#define XSI_XSD_VERSION 	"0.98"
#ifndef XSI_XSD_URI
#  define XSI_XSD_URI		"http://xsi.kolabore.ath.cx http://xsi.kolabore.ath.cx/XSD/xmlsysinfo-1.0.xsd"
#endif /* XSI_XSD_URI */

using namespace std;

/*
 * XSIBase class, the root of all evil.
 * Has class XSI_* member variables whose methods can be used to set
 * information and statistics
 */
class XSIBase : protected Util {
public:
	XSIBase(void);
	virtual ~XSIBase(void);

	void			XSIBase_init(Config *);
	void			setListenAddrs(vector<string>);
	void			setListenService(string);
	void			readSysInfo(void);
	void			updateSysInfo(void);
	void			buildSysInfoDoc(void);
	xmlDocPtr		getSysInfoDocPtr(void);

private:
	friend class XSI_TimeKeeper;

	void			updateTimeKeeper(void);

protected:
	virtual void		collectSysInfo(void);
	virtual void		updateStats(void);
	virtual void		openPrivilegedDevices(void);

	Config		       *configuration;
	xmlDocPtr		sysInfoDocument;
	vector<string>		listenAddrs;
	string			listenService;

	XSI_System		system;
	vector<Sensor *>	sensors;
	XSI_Storage		storage;
	vector<Raid>		raids;
	XSI_Network		network;
	XSI_PFilter		packetFilter;
};

#endif /* __XMLSYSINFO_H__ */
